"""
Healthcare Referral Pathways

Individual pathway modules for the healthcare referral journey.
These are defined as nested classes in journey.py but documented here for reference.
"""

# Pathways are defined as nested classes in journey.py
# This module exists for potential future standalone pathway implementations
