"""
Kaizen Orchestration Core

Core infrastructure for multi-agent orchestration.

Modules:
- patterns: Pattern definitions and base classes
- teams: Team management and agent pooling
"""

from kaizen.orchestration.core.patterns import *  # noqa: F401, F403
from kaizen.orchestration.core.teams import *  # noqa: F401, F403

__all__ = []
